/*
 *  gi1904.c
 *
 *  Copyright (c) 1999 Harald Bauer  (bauer-h@web.de)
 *
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 

#include "gi1904.h"
#include <linux/module.h>
#include <linux/poll.h>
#include <linux/malloc.h>
#include <linux/ioport.h> 
#include <linux/wrapper.h>
#include <linux/sched.h>
#define REALLY_SLOW_IO
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/segment.h>
#include <string.h>

#define MIN(a, b)	((a) < (b) ? (a) : (b))
#define MAX(a, b)	((a) > (b) ? (a) : (b))

/*#define DEBUG_GI1904 */

#ifdef DEBUG_GI1904
#define PRINTK(x...)	printk(x)
#else
#define PRINTK(x...)
#endif

static struct wait_queue *wq = NULL;
struct buffer_block_struct *start = NULL, *rd = NULL, *wr = NULL;
static char GI1904_ID[256] = "GI1904-Scanner";

/*char kernel_version[]=KERNEL_VERSION;*/	/* needed for module installer */
char kernel_version[]="2.2.5";	/* needed for module installer */

static struct device_status_struct ds = { 0, 0, 0, 0 };
static int buflines = 0, lines2read = 0, req_lines = DEFAULT_BUFLINES;
static int select_threshold = 1;
static unsigned char copy_buffer[COPY_BUFFER];
static int dithlcnt=0;
static int internal_mode=0;
static int internal_bpl;
static int rd_cp_buf_index = 0;

static char dithtab[256]={0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,
			  1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
			  1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
			  2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
			  1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
			  2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
			  2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
			  3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
			  1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
			  2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
			  2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
			  3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
			  2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
			  3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
			  3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
			  4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8};

int GI1904_IRQ = GI1904_DEFAULT_IRQ;
int GI1904_DMA = GI1904_DEFAULT_DMA;
int GI1904_IOBASE = GI1904_DEFAULT_IOBASE; 
int GI1904_SC_TYPE = GI1904_DEFAULT_SC_TYPE;
int GI1904_DMA_MASK;
int GI1904_IRQ_MASK;
int GI1904_CONTROL;

static struct modeinfo_struct modeinfo =
{
  0,0,0,0,0,0,0,0,0,0,0,0,0,-1
};

static __inline__ int dma_terminal_count(unsigned int channel)
{
  if(channel <= 3)
    return inb(DMA1_STAT_REG) & (1 << channel);
  else
    return inb(DMA2_STAT_REG) & (1 << (channel & 3));
}

static __inline__ void setup_xfer(char *addr, long int BPL)
{
  disable_dma(GI1904_DMA);
  set_dma_mode(GI1904_DMA, DMA_MODE_READ);
  clear_dma_ff(GI1904_DMA);
  set_dma_addr(GI1904_DMA, (int) addr);
  set_dma_count(GI1904_DMA, BPL);
  enable_dma(GI1904_DMA);
}

static __inline__ void scanner_standby(void)
{
  outb(STANDBY, GI1904_CONTROL);	 /* gives me a safer feeling if the */
  disable_dma(GI1904_DMA);	/* DMA channel is also disabled on the 8237 */
  ds.irq_enabled = 0;
}

static __inline__ void scanner_operate(char *addr)
{
  cli();
  setup_xfer(addr,internal_bpl);
  ds.irq_enabled = 1;
  sti();
  outb(sc_description[GI1904_SC_TYPE].operate, GI1904_CONTROL);
  enable_dma(GI1904_DMA);
}

static void gi1904_free_mem(void)
{
  struct buffer_block_struct *tmp = start, *next;
  unsigned int size = sizeof(struct buffer_block_struct);

  if (!tmp) {
    PRINTK("gi1904_free_mem: there's nothing to free\n");
    return;
  }
  scanner_standby();	/* make sure DMA xfers are really off */
  
  PRINTK("scanner is in standby-mode now\n");

  next = tmp->next;
  tmp->next = NULL;
  tmp = next;

  while (tmp) {
    next = tmp->next;
    if (tmp->data) {
      kfree_s(tmp->data, internal_bpl);
    }
    kfree_s((char*) tmp,size);
    tmp = next;
  }
  start = NULL; wr = NULL; rd = NULL;
  buflines = 0;
}

static int gi1904_get_mem(int lines)
{
  struct buffer_block_struct *tmp, **lastptr;
  unsigned int i;
  unsigned int size = sizeof(struct buffer_block_struct);

  if(lines > MAX_BUFSIZ || lines < 1)
    return -EINVAL;

  lastptr = &start;

  for (i=0;i<lines;i++) {
    PRINTK("allocate buffer_block_struct with size %d\n",size);
    tmp = (struct buffer_block_struct*)kmalloc(size, GFP_KERNEL | GFP_DMA);
    if(!tmp) {
      gi1904_free_mem();
      return -ENOMEM;
    }
    PRINTK("allocate bufline no %d with size %d\n",i,internal_bpl);
    tmp->data = kmalloc(internal_bpl, GFP_KERNEL | GFP_DMA);
    if(!tmp->data) {
      gi1904_free_mem();
      return -ENOMEM;
    }
    tmp->next = NULL;
    *lastptr = tmp;
    lastptr = &(tmp->next);
  }
  *lastptr = start;
  wr = rd = start;
  buflines = lines;

  return 0;
}
     
static void gi1904_interrupt(int irq_dummy,void *dev_id,struct pt_regs *unused_pointer_to_pt_regs_struct)
{
    PRINTK("gi1904_interrupt: begin");

    if (!ds.irq_enabled) {
       PRINTK("scanner.c: gi1904_interrupt: unexpected interrupt\n");
       return;
    }

    if (!dma_terminal_count(GI1904_DMA)) {
       PRINTK("scanner.c: gi1904_interrupt: DMA transfer not finished yet\n");
       return;
    }
    scanner_standby();
    wr = wr->next;
    scanner_operate(wr->data);
    lines2read++;
    buflines--;

    if(ds.process_sleeps) {
      PRINTK("gi1904_interrupt: the requested bytes are there, will wake up process\n");
      wake_up_interruptible(&wq);
      ds.process_sleeps = 0;
    }
}

static loff_t gi1904_lseek(struct file *file, loff_t offset, int origin)
{
  return -ESPIPE;	/* in case some luser tries to seek on a scanner... */
}

static int refill_buffer(int mode)
{ 
  int x,h,inbuffer = 0;

  switch(mode) {
  case MONO_1_BIT:
    for(x=0;x<internal_bpl;x++) 
      copy_buffer[x]=rd->data[x];
    inbuffer=x;
    break;
  case MONO_4_BIT:
    for(x=0;x<internal_bpl;x++) 
      { 
	copy_buffer[x+x]=
	  (rd->data[x]&0xf0)|(rd->data[x]>>4);
	copy_buffer[x+x+1]=
	  (rd->data[x]&0x0f)|(rd->data[x]<<4);
      }
    inbuffer=x+x;
    break;
  case MONO_6_BIT:
    for(x=0;x<internal_bpl/3;x++)
      {
	h=x+x+x; 
	copy_buffer[h+x]=
	  (rd->data[h]&0xfc)|(rd->data[h]>>6);
	copy_buffer[h+x+1]=
	  (rd->data[h]<<6)|((rd->data[h+1]>>2)&0xfc)|(rd->data[h]&0x03);
	copy_buffer[h+x+2]=
	  (rd->data[h+1]<<4)|((rd->data[h+2]>>4)&0xfc)|((rd->data[h+1]>>2)&0x03);
	copy_buffer[h+x+3]=
	  (rd->data[h+2]<<2)|((rd->data[h+2]>>4)&0x03);
      }
    inbuffer=x+x+x+x;
    break;
  case MONO_8_BIT:
    for(x=0;x<internal_bpl;x++) 
      copy_buffer[x]=rd->data[x];
    inbuffer=x;
    break;
  case MDIT_8_BIT:
    if (dithlcnt++==0) 
      for(x=0;x<internal_bpl;x++) copy_buffer[x]=0;
    for(x=0;x<internal_bpl;x++)
      { 
	copy_buffer[x]+=dithtab[rd->data[x]];
      }
    if (dithlcnt<8) break;
    dithlcnt=0;
    for(x=0;x<internal_bpl;x++) 
      copy_buffer[x]=255*copy_buffer[x]/64;
    inbuffer=x;
    break;
  case COLR_4_BIT:
    for(x=0;x<internal_bpl;x++) 
      { 
	copy_buffer[x+x]=
	  (rd->data[x]&0xf0)|(rd->data[x]>>4);
	copy_buffer[x+x+1]=
	  (rd->data[x]&0x0f)|(rd->data[x]<<4);
      }
    inbuffer=x+x;
    break;
  case COLR_6_BIT:
    for(x=0;x<internal_bpl/3;x++)
      {
	h=x+x+x; 
	copy_buffer[h+x]=
	  (rd->data[h]&0xfc)|(rd->data[h]>>6);
	copy_buffer[h+x+1]=
	  (rd->data[h]<<6)|((rd->data[h+1]>>2)&0xfc)|(rd->data[h]&0x03);
	copy_buffer[h+x+2]=
	  (rd->data[h+1]<<4)|((rd->data[h+2]>>4)&0xfc)|((rd->data[h+1]>>2)&0x03);
	copy_buffer[h+x+3]=
	  (rd->data[h+2]<<2)|((rd->data[h+2]>>4)&0x03);
      }
    inbuffer=x+x+x+x;
    break;
  case COLR_8_BIT:
    for(x=0;x<internal_bpl;x++) 
      copy_buffer[x]=rd->data[x];
    inbuffer=x;
    break;
  case CDIT_8_BIT:
    if (dithlcnt++==0) 
      for(x=0;x<internal_bpl;x++) copy_buffer[x]=0;
    for(x=0;x<internal_bpl/3;x++)
      { 
	h = x+x+x;
	copy_buffer[h]+=dithtab[(rd->data[h]&0x92)|(rd->data[h+1]&0x49)|(rd->data[h+2]&0x24)];
	copy_buffer[h+1]+=dithtab[(rd->data[h]&0x49)|(rd->data[h+1]&0x24)|(rd->data[h+2]&0x92)];
	copy_buffer[h+2]+=dithtab[(rd->data[h]&0x24)|(rd->data[h+1]&0x92)|(rd->data[h+2]&0x49)];
      }
    if (dithlcnt<8) break;
    dithlcnt=0;
    for(x=0;x<internal_bpl;x++) 
      copy_buffer[x]=255*copy_buffer[x]/64;
    inbuffer=x;
    break;
  default:PRINTK("refill_buffer: unknown bps-value !\n");
    inbuffer = -1; /* Unknown mode ! Bye ! */
  }
  return inbuffer;
}

static ssize_t gi1904_read(struct file *file, char *buf, size_t requested, loff_t *ppos)
{
  int lines_missing = 0, inbuffer=0, diff;

  if(file->f_flags & O_NONBLOCK) {
    return -EAGAIN;  /* O_NONBLOCK not supported */
  }
  if(rd_cp_buf_index == 0) {
    if (internal_mode<0) 
      lines_missing = 8;
    else
      lines_missing = 1;
    
    while (lines_missing) {
      while (lines2read < 1) {
	if(!ds.irq_enabled) {
	  PRINTK("gi1904_read: re-enabling interrupts\n");
	  scanner_operate(wr->data);
	}
	ds.process_sleeps = 1;
	interruptible_sleep_on(&wq);
	if(signal_pending(current))
	  return -EINTR;
      }
      if ((inbuffer = refill_buffer(internal_mode)) == -1) {
	PRINTK("get_available_bytes: refill_buffer failed.\n");
	return 0;
      }
      lines_missing--;
      rd = rd->next;
      lines2read--;
      buflines++;
    }
    if (requested < inbuffer) {
      copy_to_user(buf, copy_buffer , (long) requested);
      rd_cp_buf_index = requested;
      return requested;
    }
    else {
      copy_to_user(buf, copy_buffer , (long) inbuffer);
      return inbuffer;
    }
  }
  else {
    diff = modeinfo.bpl - rd_cp_buf_index;
    if (requested < diff) {
      copy_to_user(buf, &copy_buffer[rd_cp_buf_index] , (long) requested);
      rd_cp_buf_index = rd_cp_buf_index + requested;
      return requested;
    }
    else {
      copy_to_user(buf, &copy_buffer[rd_cp_buf_index] , (long) diff);
      rd_cp_buf_index = 0;
      return diff;
    }
  }
}

/* This function is evil, evil, evil.  This should be a
   _kernel_ rescheduling sleep! */
static void cold_sleep(int tics)	/* stolen from linux/drivers/net/plip.c */
{
  int start = jiffies;
  while(jiffies < start + tics)
    ; /* do nothing */
  return;
  /*current->state = TASK_INTERRUPTIBLE;
  current->timeout = jiffies + tics;
  schedule();
  return;*/
}

static int gi1904_get_mode(unsigned short* mode)
{
    int k = 0; 

    outb(0x91,GI1904_CONTROL);
    cold_sleep(10);
    while ((*mode = inb_p(GI1904_CONTROL)) & 0x80)
    {
      if (k == 10000)
	return -EIO;
      k++;
    }
    PRINTK("k: %d\n",k);
    return 0;
}

static int gi1904_set_mode(int *sc_internal_mode, struct modeinfo_struct *mode_info,const struct scanner_description* sc_desc)
{
  unsigned short gi1904_detect;
  int scan_mode=0,dpi_factor = 0;

  gi1904_get_mode(&gi1904_detect);

  switch(mode_info->sc_mode) {
    case SCM_UNKNOWN:
      mode_info->sc_mode = SCM_UNKNOWN;
      mode_info->options = -1; /* no options available */
      break;
    case SCM_MONO:
      scan_mode = MONO_1_BIT;
      mode_info->options = -1; /* no extra options available */
      break;
    case SCM_GREY:
      if (mode_info->options >= 0) {
	if (strcmp(sc_desc->gi1904_option[mode_info->options].name,GI1904_OPTION_1) == 0)
	  scan_mode = MONO_4_BIT;
	else if (strcmp(sc_desc->gi1904_option[mode_info->options].name,GI1904_OPTION_2) == 0)
	  scan_mode = MONO_6_BIT;
	else if (strcmp(sc_desc->gi1904_option[mode_info->options].name,GI1904_OPTION_3) == 0)
	  scan_mode = MONO_8_BIT;
	else if (strcmp(sc_desc->gi1904_option[mode_info->options].name,GI1904_OPTION_4) == 0)
	  scan_mode = MDIT_8_BIT;
      }
      else {
	scan_mode = SCM_UNKNOWN;
	mode_info->options = -1; /* no options available */
      }
      break;
    case SCM_TRUE:	
      if (mode_info->options >= 0) {	
	if (strcmp(sc_desc->gi1904_option[mode_info->options].name,GI1904_OPTION_1) == 0)
	  scan_mode = COLR_4_BIT;
	else if (strcmp(sc_desc->gi1904_option[mode_info->options].name,GI1904_OPTION_2) == 0)
	  scan_mode = COLR_6_BIT;
	else if (strcmp(sc_desc->gi1904_option[mode_info->options].name,GI1904_OPTION_3) == 0)
	  scan_mode = COLR_8_BIT;
	else if (strcmp(sc_desc->gi1904_option[mode_info->options].name,GI1904_OPTION_4) == 0)
	  scan_mode = CDIT_8_BIT;
      }
      else {
	scan_mode = SCM_UNKNOWN;      
	mode_info->options = -1; /* no options available */
      }
      break;
    default:
      scan_mode = SCM_UNKNOWN; 
      mode_info->options = -1; /* no options available */  
  }
  if (scan_mode != SCM_UNKNOWN)
    {
      if (sc_desc->sc_capabilities.o_xdpi & SCC_SOFTDETECT)
	{
	  dpi_factor = abs(gi1904_detect/scan_mode);
	  mode_info->xdpi = 
          mode_info->ydpi = dpi_factor*GI1904_BASE_DPI;
	}
      else
	{
	  dpi_factor = mode_info->xdpi/GI1904_BASE_DPI;
	}
      internal_bpl = abs(sc_desc->pix_factor*scan_mode*dpi_factor/8);
      switch(scan_mode) {
      case MONO_4_BIT:
	mode_info->bpl = 2*internal_bpl;
	break; 
      case MONO_6_BIT:
	mode_info->bpl = 4*internal_bpl/3; 
	break; 
      case COLR_4_BIT:
	mode_info->bpl = 2*internal_bpl; 
	break; 
      case COLR_6_BIT:
	mode_info->bpl = 4*internal_bpl/3; 
	break; 
      default:
	mode_info->bpl = internal_bpl; 
      }
    }
  *sc_internal_mode = scan_mode;
  return 0;
}

static int gi1904_ioctl(struct inode *inode, struct file *file, \
	unsigned int cmd, unsigned long arg)
{
  signed short av_options=0;

  if(!arg)
    return -EINVAL;
    
  switch(cmd) {
    case HSIOCGOPT:			/* get scanner_options struct */
      av_options = sc_description[GI1904_SC_TYPE].sc_capabilities.av_options;
      copy_to_user((char*) arg,(char*) sc_description[GI1904_SC_TYPE].gi1904_option, \
                   av_options * (long) sizeof(struct scanner_option));
      return 0;
    case HSIOCGSCC:			/* get scanner_capabilities struct */
      copy_to_user((char*) arg, (char*) &sc_description[GI1904_SC_TYPE].sc_capabilities, (long) \
                   sizeof(struct scanner_capabilities ));
      return 0;

    case HSIOCGMOD:			/* get modeinfo */
      copy_to_user((char*) arg, (char*) &modeinfo, (long) sizeof(struct modeinfo_struct));
      return 0;

    case HSIOCSMOD:			/* set modeinfo struct */
      gi1904_free_mem();
      copy_from_user((char*) &modeinfo, (char*) arg, (long) sizeof(struct modeinfo_struct));
      gi1904_set_mode(&internal_mode, &modeinfo, &sc_description[GI1904_SC_TYPE]);
      gi1904_get_mem(req_lines);
      return 0;

    case HSIOCGSCT:                     /* get scanner_type struct */
      copy_to_user((char*) arg, (char*) &sc_description[GI1904_SC_TYPE].sc_type, (long) \
                   sizeof(struct scanner_type ));
      return 0;

    case HSIOCGREG:			/* get setting of misc registers */
      return 0;

    case HSIOCSREG:			/* set misc registers */
      return 0;

    case HSIOCSBUF:			/* set size of internal buffer */
      gi1904_free_mem();
      get_user ((unsigned long) req_lines,(unsigned long*) arg);
      gi1904_get_mem(req_lines);
      return 0;	

    case HSIOCGBUF:			/* get current buffer size */
      put_user ((unsigned long) req_lines,(unsigned long*) arg);
      return 0;

    case HSIOCSSTH:			/* set select(2) threshold */
      get_user(select_threshold,(int*) arg);
      return 0;

    case HSIOCGSTH:			/* get current select(2) threshold */
      put_user(select_threshold,(int*) arg);
      return 0;
      
    case HSIOCGSIB:			/* get number of lines in buffer */
      put_user ((unsigned long) req_lines-buflines,(unsigned long*) arg);
      return 0;

    default:
      return -ENOSYS;
  }
}

static int gi1904_open(struct inode *inode, struct file *file)
{
  int r;
  
  PRINTK("gi1904_open: begin opening device\n");

  if(MINOR(inode->i_rdev) != 0)
    return -ENODEV;

  if(ds.device_open)
    return -EBUSY;

  ds.irq_enabled = 0; 
  ds.process_sleeps = 0;

  outb_p(OFF, GI1904_CONTROL);	 /* interrupts and DMA transfers are disabled */

  cold_sleep(10);

  outb_p(GI1904_IRQ_MASK | GI1904_DMA_MASK, GI1904_IOBASE); /* set irq & DMA mask _before_ */

  PRINTK("gi1904_open: requesting interrupt\n");
  if((r = request_irq(GI1904_IRQ, gi1904_interrupt,SA_INTERRUPT,"GI1904-Scanner",GI1904_ID))) {  
    outb_p(0, GI1904_IOBASE); /* clear irq & DMA mask _before_ */
    outb(OFF, GI1904_CONTROL); 		
    return r;
  }

  PRINTK("gi1904_open: requesting dma\n");
  if((r = request_dma(GI1904_DMA,GI1904_ID))) {
    outb(OFF, GI1904_CONTROL);
    outb_p(0, GI1904_IOBASE); /* clear irq & DMA mask _before_ */
    free_irq(GI1904_IRQ,GI1904_ID);
    return r;
  }

  scanner_standby();
  lines2read = 0;
  buflines = 0;

  /* initialize modeinfo struct on every open */
  memset(&modeinfo, 0, sizeof(struct modeinfo_struct));
  modeinfo.options = -1;
 
  MOD_INC_USE_COUNT;
  ds.device_open = 1;
  PRINTK("gi1904_open: end opening device\n");
  return 0;	/* device open, scanner is in standby mode now */
}

static int gi1904_release(struct inode *inode, struct file *file)
{
  scanner_standby();
 
  gi1904_free_mem();

  outb_p(0, GI1904_IOBASE); /* clear irq & DMA mask _before_ */
  outb(OFF, GI1904_CONTROL);
  free_dma(GI1904_DMA);
  free_irq(GI1904_IRQ,GI1904_ID);
  MOD_DEC_USE_COUNT;
  ds.device_open = 0;
  return 0;
}

static struct file_operations gi1904_fops = {
  gi1904_lseek,
  gi1904_read,
  NULL,		/* write */
  NULL,		/* readdir */
  NULL,         /* poll */
  gi1904_ioctl,
  NULL,		/* mmap */
  gi1904_open,
  NULL,
  gi1904_release,
};

static int get_scanner_parameters(void)
{ 
  if (GI1904_DMA != 1 && GI1904_DMA != 3)
    {PRINTK("Sorry,only DMA 1 and 3 are supported.\n");return(-ENODEV);}
  
  if (GI1904_IRQ != 3 && GI1904_IRQ != 5 && GI1904_IRQ != 10)
    {PRINTK("Sorry,only IRQ 3,5 and 10 are supported.\n");return(-ENODEV);}
   
  if (GI1904_IOBASE != 0x22b && GI1904_IOBASE != 0x26b && GI1904_IOBASE != 0x2ab &&\
      GI1904_IOBASE != 0x2eb && GI1904_IOBASE != 0x32b && GI1904_IOBASE != 0x36b &&\
      GI1904_IOBASE != 0x3ab && GI1904_IOBASE != 0x3eb)
    {PRINTK("Sorry,you have to use one of these IO-addresses: 0x22b, 0x26b, 0x2ab, 0x2eb,0x32b, 0x36b, 0x3ab or 0x3eb\n");return(-ENODEV);}
  
  if (GI1904_SC_TYPE != 0 && GI1904_SC_TYPE != 1)
    {PRINTK("Sorry,only the values 0 and 1 are supported\n");return(-ENODEV);}

  PRINTK("loaded gi1904 with the following parameters:\nGI1904_IOBASE: 0x%x\nGI1904_IRQ: %d\nGI1904_DMA: %d\nGI1904_SC_TYPE: %d\n",GI1904_IOBASE,GI1904_IRQ,GI1904_DMA,GI1904_SC_TYPE);
  
  if (GI1904_DMA == 1)
    GI1904_DMA_MASK = 0x01;
  else
    GI1904_DMA_MASK = 0x04;

  switch (GI1904_IRQ) {
    case 3:
      GI1904_IRQ_MASK = 0x10;
      break;
    case 5:
      GI1904_IRQ_MASK = 0x20;
      break;
    case 10:
      GI1904_IRQ_MASK = 0x08;
  }
  PRINTK("%x %x\n",GI1904_DMA_MASK,GI1904_IRQ_MASK);
  
  return(0);
}

int init_module(void)
{
  printk("gi1904: GI1904 handheld scanner driver  version 0.2\n");

  if (register_chrdev(GI1904_SCANNER_MAJOR, GI1904_SCANNER_NAME, &gi1904_fops)) {
    PRINTK("gi1904_init: cannot register major number\n");
    return -EIO;
  }

  if (get_scanner_parameters())
  { PRINTK("gi1904_init: scanner interface not found\n");
    return -EIO;
  }
  GI1904_CONTROL = GI1904_IOBASE + 1;
  request_region(GI1904_IOBASE,5,GI1904_ID);
  outb_p(OFF, GI1904_CONTROL);
  return 0;
}

void cleanup_module(void)
{
  printk("gi1904: Removing GI1904 handheld scanner driver from memory\n");

  release_region(GI1904_IOBASE,5);

  if (MOD_IN_USE)
    PRINTK("gi1904_cleanup: device is busy, delaying driver removal\n");

  if (unregister_chrdev(GI1904_SCANNER_MAJOR, GI1904_SCANNER_NAME))
    PRINTK("gi1904_cleanup: unregister_chrdev() failed\n");
}




