/*
 *  gi1904.h
 *
 *  Copyright (c) 1999 Harald Bauer  (bauer-h@web.de)
 *
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 

#ifndef _GI1904_H
#define _GI1904_H

#include <scanner.h>

#define GI1904_SCANNER_NAME	"Scanner"
#define GI1904_SCANNER_MAJOR	144	/* our major number */
#define MAX_BUFSIZ		200     /* max bufferlines */
#define DEFAULT_BUFLINES        100
#define STANDBY			0x05 
#define OFF			0
#define GI1904_DEFAULT_IOBASE		0x3eb	/* default base address of scanner interface
					           (0x22b, 0x26b, 0x2ab, 0x2eb,
					            0x32b, 0x36b, 0x3ab or 0x3eb) */
#define GI1904_DEFAULT_IRQ		10	/* default IRQ  (3, 5 or 10) */
#define GI1904_DEFAULT_DMA		3	/* default DMA channel  (1 or 3) */
#define GI1904_DEFAULT_SC_TYPE          1

#define MONO_1_BIT 1
#define MONO_4_BIT 4
#define MONO_6_BIT 6
#define MONO_8_BIT 8
#define MDIT_8_BIT -8
#define COLR_4_BIT 12
#define COLR_6_BIT 18
#define COLR_8_BIT 24
#define CDIT_8_BIT -24

/* gi1904 options */

#define GI1904_OPTION_1 "4-bit-per-channel"
#define GI1904_OPTION_2 "6-bit-per-channel"
#define GI1904_OPTION_3 "8-bit-per-channel"
#define GI1904_OPTION_4 "8-bit-dithering"

#define COPY_BUFFER 10000	/* Copy_Buffer_Size must be big enough
				   for any ONE line in any MODE */

#define GI1904_BASE_DPI 50

struct scanner_description {
  unsigned short operate;
  int pix_factor;
  struct scanner_type sc_type;
  struct scanner_capabilities sc_capabilities;
  const struct scanner_option *gi1904_option;
};

struct device_status_struct {
  char device_open:	1;
  char irq_enabled:	1;
  char process_sleeps:	1;
  char buffer_empty:	1;
};

struct buffer_block_struct {
  struct buffer_block_struct *next;
  unsigned char *data;
};

/* scanner-type descriptions */

static const struct scanner_option m105_options[1] =
{
  {SCC_HARDSWITCH,GI1904_OPTION_3}
};

static const struct scanner_option CG_6000_options[4] =
{
  {SCC_HARDSWITCH,GI1904_OPTION_1},
  {SCC_HARDSWITCH,GI1904_OPTION_2},
  {SCC_HARDSWITCH,GI1904_OPTION_3},
  {SCC_HARDSWITCH,GI1904_OPTION_4}
};

static const struct scanner_description sc_description[2] =
{
  {
    0x91,
    208,
    { "Mustek","m105" },
    { 
      SCC_HARDSWITCH|SCC_SOFTSELECT,             /* Options for monochrome mode */
      SCC_HARDSWITCH|SCC_SOFTSELECT|SCC_EMULATE, /* Options for greyscale  mode */
      0,                                         /* Options for true-color mode */
      SCC_HARDSWITCH|SCC_SOFTDETECT,             /* Options for x resolution */
      SCC_HARDSWITCH|SCC_SOFTDETECT,             /* Options for y resolution */
      SCC_HARDSWITCH,                            /* Options for brightness */
      {0,0},
      0,                                         /* Options for contrast */
      {0,0},
      0,                                         /* Options for hue */
      {0,0},
      0,                                         /* Options for saturation */
      {0,0},
      105,                                       /* Width in mm */
      0,                                         /* Length in mm (unlimited) */
      1                                          /* 1 options available: 8-bit-dithering*/
    },
    &m105_options[0]
  },
  {
    0x91,
    208,
    { "Mustek","CG-6000" },
    { 
      SCC_HARDSWITCH|SCC_SOFTSELECT,                 /* Options for monochrome mode */
      SCC_HARDSWITCH|SCC_SOFTSELECT|SCC_EMULATE,     /* Options for greyscale  mode */
      SCC_HARDSWITCH|SCC_SOFTSELECT|SCC_EMULATE,     /* Options for true-color mode */
      SCC_HARDSWITCH|SCC_SOFTDETECT,                 /* Options for x resolution */
      SCC_HARDSWITCH|SCC_SOFTDETECT,                 /* Options for y resolution */
      SCC_HARDSWITCH,                                /* Options for brightness */
      {0,0},
      0,                                             /* Options for contrast */
      {0,0},
      0,                                             /* Options for hue */
      {0,0},
      0,                                             /* Options for saturation */
      {0,0},
      105,                                           /* Width in mm */
      0,                                             /* Length in mm (unlimited) */
      4                                              /* 4 options available: 
							4-bit-per-channel,6-bit-channel,
							8-bit-channel,8-bit-dithering*/
    },
    &CG_6000_options[0]
  }
};

#endif /* _GI1904_H */












