/*
 *  scanfunc.c
 *
 *  Copyright (c) 1999 Harald Bauer  (bauer-h@web.de)
 *  
 *  This is a litle commandline-interface for the gi1904-driver.
 *  It has been tested with my mustek CG-6000 handscanner.
 *  
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <scanner.h>

int get_scanner_options(int fd,struct scanner_option *options)
{  
   if (ioctl(fd,HSIOCGOPT,options) == -1)
   {
     perror("get scanner options");
     return 0;
   }
   return 1; 
}

int get_scanner_capabilities(int fd,struct scanner_capabilities *capabilities)
{       
   if (ioctl(fd,HSIOCGSCC,capabilities) == -1)
   {
      perror("get scanner capabilities");
      return 0;
   }
   return 1; 
}

int get_scanner_mode(int fd,struct modeinfo_struct *mode)
{       
   if (ioctl(fd,HSIOCGMOD,mode) == -1)
   {
      perror("get scanner mode");
      return 0;
   }
   return 1; 
}

int set_scanner_mode(int fd,struct modeinfo_struct *mode)
{       
   if (ioctl(fd,HSIOCSMOD,mode) == -1)
   {
      perror("set scanner mode");
      return 0;
   }
   return 1; 
}

int get_scanner_type(int fd,struct scanner_type *type)
{       
   if (ioctl(fd,HSIOCGSCT,type) == -1)
   {
      perror("get scanner type");
      return 0;
   }
   return 1; 
}

int get_scanlines_in_buffer(int fd,unsigned long *buffer)
{       
   if (ioctl(fd,HSIOCGSIB,buffer) == -1)
   {
      perror("get scanlines in buffer");
      return 0;
   }
   return 1; 
}

int get_data(int fd,char *header,int ydim,struct modeinfo_struct *mode)
{
   int byte_per_line,i=0;
   time_t t,t_tmp;
   unsigned char buf[4096];
   unsigned long buffer;
   FILE* fp;

   if(!(fp=fopen("raw.tmp","wb")))
     perror("open tmp-file");
   byte_per_line = mode->bpl;
   fprintf(fp,"%s",header);
   time(&t);
   for(i=0;i<ydim;i++)
   {
      if(read(fd,buf,byte_per_line)>0)
         fwrite(buf,byte_per_line,1,fp);
      else
      {
         printf("Lesefehler: EXIT\n");
         break;
      }
      get_scanlines_in_buffer(fd,&buffer);
      printf("lines in buffer: %ld\r",buffer);
      fflush(stdout);
      /*if ((i % 100) == 0)
      {
	time(&t_tmp);
	printf("line number: %d sec: %ld\r",i,t_tmp-t);
	if ((t_tmp-t) == 0)
	  printf("\a");
        t=t_tmp;
	}*/
   }
   fclose(fp);
    
   return 1;
} 

int main(int argc,char** argv)
{
   int i,fd=0,xdim,ydim;
   char buf[80],header[20],str_end='\0'; 
   struct modeinfo_struct mode;
   struct scanner_option options[3];
   struct scanner_capabilities capabilities;
   struct scanner_type type;

   /* if((argc < 3) || (argc > 3))
   {
      printf("usage: %s <number of scanlines> <name of the scanimage>\n",argv[0]);
      exit(1);
   }
   ydim = atoi(argv[1]);*/

   if (fd < 1)
      fd = open("/dev/scan",O_RDONLY);
      
   if (fd < 1)
      return 0;
   
   get_scanner_type(fd,&type);
   printf("The following Scanner is connected to the device:\n");
   printf("manufacturer: %s\nmodel: %s\n",type.manufacturer,type.model);

   get_scanner_capabilities(fd,&capabilities);
   printf("The folowing modes are supported by the Scanner:\n");
   if (capabilities.o_scm_mono & (SCC_SOFTSELECT|SCC_HARDSWITCH))
     printf("mono-mode (1)\n");
   if (capabilities.o_scm_grey & (SCC_SOFTSELECT|SCC_HARDSWITCH))
     printf("grey-mode (2)\n");
   if (capabilities.o_scm_true & (SCC_SOFTSELECT|SCC_HARDSWITCH))
     printf("true-mode (3)\n");

   get_scanner_options(fd,options);
   printf("The folowing options are supported by the Scanner:\n");
   for (i=0;i<capabilities.av_options;i++)
     printf("Option: %s (%d)\n",options[i].name,i);
 
   printf ("choose one of the modes with optional one of the options\n");
   printf ("mode: ");
   switch (atoi(gets(buf))) {
     case 1:
       mode.sc_mode = SCM_MONO;
       break;
     case 2:
       mode.sc_mode = SCM_GREY;
       break;
     case 3:
       mode.sc_mode = SCM_TRUE;
       break;
     default:
       mode.sc_mode = SCM_UNKNOWN;
   }
     
   printf ("option: ");
   mode.options = atoi(gets(buf));

   printf("choose the Number of scanlines:\n");
   ydim=atoi(gets(buf));
   mode.xdpi=0;
   set_scanner_mode(fd,&mode);
   get_scanner_mode(fd,&mode);
   switch (mode.sc_mode) {
     case SCM_MONO:
       xdim=8*mode.bpl;
       sprintf(header,"P4\n%d %d\n%c",xdim,ydim,str_end);
       break;
     case SCM_GREY:
       xdim=mode.bpl;
       sprintf(header,"P5\n%d %d\n255\n%c",xdim,ydim,str_end);
       break;
     case SCM_TRUE:
       xdim=mode.bpl/3;
       sprintf(header,"P6\n%d %d\n255\n%c",xdim,ydim,str_end);
       break;
     default:
   }
   printf("sc_mode: %d\nxdpi: %d\nbpl: %d\n",mode.sc_mode,mode.xdpi,mode.bpl);
   printf("sc_option: %s\n",options[mode.options].name);
   if (mode.sc_mode == SCM_UNKNOWN)
     return 0;
   get_data(fd,header,ydim,&mode);
   close(fd);

   return 1;
}













